# models.py in Dispatch app
from django.db import models
from django.utils.crypto import get_random_string

from account.models import Account
from profiling.models import ProfilingJob
from quotations.models import Quotation
from transport.models import Driver, Vehicle  # Importing Driver and Vehicle models

class DispatchProduct(models.Model):
    STATUS_CHOICES = [
        ('Dispatched', 'Dispatched'),
        ('In Transit', 'In Transit'),
        ('Received', 'Received'),
        ('Completed', 'Completed'),
    ]

    driver = models.ForeignKey(Driver, on_delete=models.CASCADE)
    vehicle = models.ForeignKey(Vehicle, on_delete=models.CASCADE)
    transport_cost = models.DecimalField(max_digits=10, decimal_places=2)
    dispatch_date = models.DateField()
    expected_delivery_date = models.DateField()
    quotation = models.ForeignKey(Quotation, on_delete=models.CASCADE)
    dispatched_by = models.ForeignKey(Account, on_delete=models.CASCADE)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='Dispatched')
    files = models.FileField(upload_to='dispatch_files/', null=True, blank=True)
    def __str__(self):
        return f"Dispatch {self.id} - {self.quotation.qid}"
    
class Dispatch(models.Model):
    DISPATCH_STATUS_CHOICES = [
        ('Pending', 'Pending'),
        ('Dispatched', 'Dispatched'),
        ('Delivered', 'Delivered'),
    ]

    DISPATCH_TYPES = [
        ('Profiling', 'Profiling'),
        ('Production', 'Production'),
    ]

    track_number = models.CharField(max_length=8, unique=True)
    transport_cost = models.DecimalField(max_digits=10, decimal_places=2)
    dispatch_date = models.DateField()
    expected_delivery_date = models.DateField()
    dispatched_by = models.ForeignKey(Account, on_delete=models.CASCADE)
    status = models.CharField(max_length=20, choices=DISPATCH_STATUS_CHOICES, default='Pending')

    # Foreign key relationships
    vehicle = models.ForeignKey(Vehicle, on_delete=models.SET_NULL, null=True)  # Linking Vehicle
    driver = models.ForeignKey(Driver, on_delete=models.SET_NULL, null=True)  # Linking Driver
    quotation = models.ForeignKey(Quotation, on_delete=models.CASCADE, null=True, blank=True)
    dispatch_type = models.CharField(max_length=10, choices=DISPATCH_TYPES, default='Production')

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    profiling_job = models.ForeignKey(ProfilingJob, on_delete=models.SET_NULL, null=True, blank=True)
    collected_by = models.CharField(max_length=255, blank=True, null=True)

    def save(self, *args, **kwargs):
        if not self.track_number:
            self.track_number = self.generate_track_number()
        super().save(*args, **kwargs)

    def generate_track_number(self):
        return 'QD' + get_random_string(length=6, allowed_chars='ABCDEFGHIJKLMNOPQRSTUVWXYZ')

    def add_image(self, image_file):
        DispatchImage.objects.create(dispatch=self, image=image_file)


class DispatchImage(models.Model):
    dispatch = models.ForeignKey(Dispatch, related_name='images', on_delete=models.CASCADE)
    image = models.ImageField(upload_to='dispatch_images/')

    def __str__(self):
        return f"Image for {self.dispatch.track_number}"
